import maya.cmds as mc
import os
import re
import logging

# Constants
PLUGIN_BLACKLIST = [
    "LookdevXMaya",
    "CoreWeave",
    "Conductor",
    "mayaUsdPlugin",
    "mayaVnnPlugin",
]

# Configure logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

def get_hardware_hints_model():
    """
    Returns a model of hardware hints.
    
    Returns:
        list: A list of dictionaries containing hardware hints.
    """
    return [
        {
            "label": "Hints", 
            "order": 0,
            "content": [
                {"name":"CLI", "description": "CLI - Best for simple shell commands"},
                {"name":"Compositor", "description": "Compositor - Best for image processing"},
                {"name":"CPU-render", "description": "CPU-render - Best for software rendering"},
                {"name":"Simulation", "description": "Simulation - Best for memory intensive tasks"},
                {"name":"GPU-render", "description": "GPU-render - Best for hardware rendering"},
            ],
         }
    ]

def detect_session_projects_model():
    """
    Returns a model of projects.

    This function fetches the projects from coredata and returns a model containing the project names and descriptions.

    Returns:
        list: A list of dictionaries, where each dictionary represents a project with the keys 'name' and 'description'.
    """
    try:
        projects = [os.path.basename(p) for p in mc.optionVar(q='RecentProjectsList') or ["default"]]
    except Exception as e:
        logger.error("Failed to fetch recent projects: %s", e)
        projects = ["default"]
    return [{"name": p, "description": p} for p in projects]

def detect_session_software_model():
    """
    Detects and returns a model of the current session's software and plugins.

    The path for the Maya installation directory. If it is not set, it defaults to
    /usr/autodesk/MayaVersionNumber (Linux) or C:\Program Files\Autodesk \MayaVersionNumber (Windows).

    On macOS, Maya is installed as /Applications/Maya <version>/Maya.app. 

    Returns:
        list: A list of dictionaries, where each dictionary represents a software or plugin with the keys 'name' and 'description'.
    """
    
    maya_location_regex = re.compile(r'.*[/\\]Autodesk[/\\]Maya')
    if mc.about(query=True, mac=True):
        maya_location_regex = re.compile(r'^.*/Autodesk/maya')
    elif mc.about(query=True, linux=True):
        maya_location_regex = re.compile(r'.*/autodesk/maya')
        

        
    maya_api_version = mc.about(query=True, apiVersion=True)
    year = str(maya_api_version)[:4]
    sp = str(int(str(maya_api_version)[4:6]))  # Convert to int to remove padding
    host_name = f"maya-{maya_api_version}"
    host_description = f"Maya {year} SP{sp}"

    try:
        loaded_plugin_paths = mc.pluginInfo(query=True, listPluginsPath=True)
        loaded_plugins = mc.pluginInfo(query=True, listPlugins=True)
    except Exception as e:
        logger.error("Failed to query plugins: %s", e)
        return []

    zipped = zip(loaded_plugins, loaded_plugin_paths)

    result = [
        {
            "name": host_name,
            "description": host_description,
        }
    ]

    for plugin, path in zipped:
        # print(maya_path_pattern, path)
        if maya_location_regex.match(path) or plugin in PLUGIN_BLACKLIST:
            continue
        try:
            version = mc.pluginInfo(plugin, query=True, version=True)
        except Exception as e:
            logger.warning("Failed to get version for plugin %s: %s", plugin, e)
            version = "unknown"
        logger.info("Plugin loaded: %s %s %s", plugin, version, path)
        result.append({
            "name": f"{host_name}|{plugin}-{version}",
            "description": f"{plugin} {version}",
        })

    return result

def hydrate_data_models(dialog):
    """
    Initializes core data with predefined services and attempts to populate the given dialog's tabs with instance types, package, and project models.

    Args:
        dialog: The dialog object containing tabs that need to be updated with the core data.

    Returns:
        None: The function does not return a value but updates the dialog components in place.
    """
    kwargs = {
        "packages_model": detect_session_software_model(),
        "projects_model": detect_session_projects_model(),
        "hardware_model": get_hardware_hints_model(),
    }
    for tab in dialog.tabs.values():
        tab.hydrate_data_models(**kwargs)
