import sys
import os
import maya.api.OpenMaya as om
import maya.cmds as cmds


CIODIR = os.environ.get("CWMAYA_CIODIR")
sys.path.append(CIODIR)

from cwmaya.template.base.node import cwSubmissionNode
from cwmaya.template.flat.node import cwFlatNode
from cwmaya.template.sim_render.node import cwSimRenderNode
from cwmaya.template.chain.node import cwChainNode
from cwmaya.menus import coreweave_menu
from cwmaya.helpers import preset_helpers



from cwmaya.logger_config import configure_logger

logger = configure_logger()

def some_conductor_function():
    logger.info("This is an info message from Conductor.")
    logger.error("This is an error message from Conductor.") 

def maya_useNewAPI():
    pass


def initializePlugin(obj):
    print("Initializing CoreWeave plugin")
    plugin = om.MFnPlugin(obj, "CoreWeave", "0.0.3", "Any")
    print("Got MFnPlugin")
    try:
        plugin.registerNode(
            "cwSubmissionNode",
            cwSubmissionNode.id,
            cwSubmissionNode.creator,
            cwSubmissionNode.initialize,
            om.MPxNode.kDependNode,
        )

        print("Registered cwSubmissionNode")

        plugin.registerNode(
            "cwFlatNode",
            cwFlatNode.id,
            cwFlatNode.creator,
            cwFlatNode.initialize,
            om.MPxNode.kDependNode,
        )

        print("Registered cwFlatNode")

        plugin.registerNode(
            "cwSimRenderNode",
            cwSimRenderNode.id,
            cwSimRenderNode.creator,
            cwSimRenderNode.initialize,
            om.MPxNode.kDependNode,
        )

        print("Registered cwSimRenderNode")

        plugin.registerNode(
            "cwChainNode",
            cwChainNode.id,
            cwChainNode.creator,
            cwChainNode.initialize,
            om.MPxNode.kDependNode,
        )

        print("Registered cwChainNode")

        create_storm_shelf()
        print("Created storm shelf")

        create_file_rules()
        print("Created file rules")

        install_presets()
        print("Installed presets")

        print("Initialized plugin")
    except:
        sys.stderr.write("Failed to register submission nodes\n")
        raise

    coreweave_menu.load()


def uninitializePlugin(obj):
    plugin = om.MFnPlugin(obj)

    try:
        remove_storm_shelf()
        plugin.deregisterNode(cwChainNode.id)
        plugin.deregisterNode(cwSimRenderNode.id)
        plugin.deregisterNode(cwFlatNode.id)
        plugin.deregisterNode(cwSubmissionNode.id)
    except:
        sys.stderr.write("Failed to deregister submission nodes\n")
        raise

    coreweave_menu.unload()


# We'll move this shelf stuff to its own module.
def create_storm_shelf():
    shelf_name = "Conductor"

    # Delete the shelf if it already exists
    if cmds.shelfLayout(shelf_name, exists=True):
        cmds.deleteUI(shelf_name, layout=True)

    # Create a new shelf
    cmds.shelfLayout(shelf_name, parent="ShelfLayout")

    # Add buttons to the shelf
    cmds.shelfButton(
        label="My Button",
        command='print("Button Pressed")',
        image1="commandButton.png",  # Path to your icon file
        parent=shelf_name,
    )

    cmds.shelfButton(
        label="Conductor Window",
        command="from cwmaya.menus.coreweave_menu import show_conductor_window;show_conductor_window()",
        image1="commandButton.png",  
        parent=shelf_name,
    )

    cmds.shelfButton(
        label="Desktop Health",
        command="from cwmaya.helpers import desktop_app_helpers;desktop_app_helpers.health_check()",
        image1="da_health_128x128.png",
        parent=shelf_name,
    )

    cmds.shelfButton(
        label="Workflow API Health",
        command="from cwmaya.helpers import workflow_api_helpers;workflow_api_helpers.health_check()",
        image1="wf_health_128x128.png",
        parent=shelf_name,
    )


    cmds.shelfButton(
        label="Select current template",
        command="from cwmaya.helpers import spec_helpers;from cwmaya.windows import window;win = spec_helpers.select_current_template(window.ConductorWindow.get_instance())",
        image1="tp_select_128x128.png",
        parent=shelf_name,
    )


def remove_storm_shelf():
    shelf_name = "Conductor"
    if cmds.shelfLayout(shelf_name, exists=True):
        cmds.deleteUI(shelf_name, layout=True)


def create_file_rules():
    cmds.workspace(fileRule=["storm", "storm/"])
    cmds.workspace(fileRule=["storm_cache", "storm/cache"])
    cmds.workspace(fileRule=["storm_scenes", "storm/scenes"])
    cmds.workspace(fileRule=["storm_comp", "storm/comp"])


def install_presets():
    """
    Install the presets that the developer has set up and included in the distribution.

    Should include a default preset for each node type, so that the new nodes are hydrated on creation.

    Only presets if that don't already exist in the user's presets folder.
    """
    preset_helpers.install_presets()
