# -*- coding: utf-8 -*-

"""
This module defines a UI control for managing a list of software selections within Maya.

The SoftwareStackControl provides a list interface where users can add/remove software selections from a predefined model. Each row contains a dropdown menu showing available software options and a delete button.

The model is a list of dictionaries, where each dictionary has a "name" field (used as the unique identifier) and a "description" field (shown in the UI):

[
    {
        "name": "unique_identifier1", 
        "description": "User Friendly Name 1"
    },
    {
        "name": "unique_identifier2",
        "description": "User Friendly Name 2" 
    }
]

The control follows this workflow:

1. Bind: Associates the control with a Maya array attribute that stores the selected software names
   - self.attribute = attribute

2. Hydrate: Populates the UI with the current attribute values using the provided model
   - self.model = model
   - Creates a row for each existing attribute element
   - Each row shows a dropdown with descriptions from the model
   - Selected values are preserved if they exist in the model
   - Invalid values are removed

3. User Interactions:
   - Add: Creates a new attribute element and row with the first unused model entry
   - Remove: Deletes the row and removes the attribute element
   - Change: Updates the attribute element when a different option is selected
   - The UI prevents duplicate selections by disabling used options in dropdowns

The control maintains consistency between the UI state, attribute values, and model data.
"""

import pymel.core.uitypes as gui
import pymel.core as pm
import cwmaya.helpers.const as k
from cwmaya.helpers import node_utils

ADD_ICON = "item_add.png"
REMOVE_ICON = "item_delete.png"

class SoftwareStackControl(gui.FormLayout):
    """
    A custom form layout control designed to manage a list of software selections.

    Attributes:
        model (list): List of dictionaries containing software options with name/description pairs
        attribute: The Maya array attribute storing the selected software names
        header_row: The row layout containing the header and add button
        column: The column layout containing the software selection rows
        add_btn: The button control for adding new entries
    """

    def __init__(self):
        """
        Initializes the SoftwareStackControl instance, creating its UI components.
        """
        super(SoftwareStackControl, self).__init__()
        self.model = None
        self.attribute = None
        self.header_row = None

        self.host_label_ctl = None
        self.host_menu = None

        self.plugins_column = None
        self.add_btn = None
        self.build_ui()

    def build_ui(self):
        """
        Constructs the UI elements for the software stack control and sets up their layout.
        """
        pm.setParent(self)

        self.label = pm.text(label="", width=k.LABEL_WIDTH)

        self.header_row = pm.rowLayout(nc=2, adjustableColumn=1)
        self.header_text = pm.text(
            align="left",
            ebg=True,
            bgc=k.LIST_HEADER_BG,
            label=" Software",
            height=24,
        )
        self.add_btn = pm.symbolButton(
            image=ADD_ICON,
            width=k.TRASH_COLUMN_WIDTH,
            height=24,
            command=pm.Callback(self.on_add_entry),
        )
        pm.setParent(self)

        self.column = pm.columnLayout(adj=True)

        self.attachForm(self.label, "left", k.FORM_SPACING_X)
        self.attachNone(self.label, "right")
        self.attachForm(self.label, "top", k.FORM_SPACING_Y)
        self.attachForm(self.label, "bottom", k.FORM_SPACING_Y)

        self.attachControl(self.header_row, "left", k.FORM_SPACING_X, self.label)
        self.attachForm(self.header_row, "right", k.FORM_SPACING_X)
        self.attachForm(self.header_row, "top", k.FORM_SPACING_Y)
        self.attachNone(self.header_row, "bottom")

        self.attachControl(self.column, "left", k.FORM_SPACING_X, self.label)
        self.attachForm(self.column, "right", k.FORM_SPACING_X)
        self.attachControl(self.column, "top", 0, self.header_row)
        self.attachForm(self.column, "bottom", k.FORM_SPACING_Y)

    def bind(self, attribute):
        """
        Associates the control with a Maya array attribute.

        Args:
            attribute: The Maya array attribute to bind to the control.
        """
        self.attribute = attribute

    def on_add_entry(self):
        """
        Callback function that adds a new entry to the software stack control.
        """
        # Get all currently used software names
        used_names = set(self.attribute.get())

        # Find first unused model entry
        new_entry = next(
            (entry for entry in self.model if entry["name"] not in used_names),
            None,  # Fallback if all are used
        )
        if not new_entry:
            return

        attr_element = node_utils.next_element_plug(self.attribute)
        attr_element.set(new_entry["name"])
        self.create_row(attr_element)
        self.enable_disable_add_button()

    def create_row(self, attr_element):
        """
        Creates a new row in the plugin column for a given attribute element.

        At this point, the model is valid.

        Args:
            attr_element: The attribute element for which to create the row.

        Returns:
            The newly created form layout for the row.
        """
        value = attr_element.get()
        entry = self._find_model_entry(value)
        if not entry:
            return

        pm.setParent(self.column)
        menu = pm.optionMenu()
        pm.optionMenu(
            menu, edit=True, pmc=pm.Callback(self._enable_disable_menu_items, menu)
        )
        del_ctl = pm.symbolButton(image=REMOVE_ICON, width=k.TRASH_COLUMN_WIDTH)
        row = _form_layout(menu, del_ctl)
        pm.symbolButton(
            del_ctl,
            edit=True,
            command=pm.Callback(self.remove_entry, attr_element, row),
        )

        pm.setParent(menu, menu=True)
        menu.clear()
        for item in self.model:
            pm.menuItem(label=item["description"])

        menu.setValue(entry["description"])

        menu.changeCommand(pm.Callback(self.on_menu_change, attr_element, menu))

        return row

    def _enable_disable_menu_items(self, menu):
        """
        Enables or disables menu items based on used entries.
        """
        attribute_values = set(self.attribute.get())
        for menu_item in pm.menu(menu, q=True, itemArray=True) or []:
            label = pm.menuItem(menu_item, q=True, label=True)
            model_entry = self._find_model_entry(label)
            pm.menuItem(menu_item, edit=True, enable=model_entry.get("name") not in attribute_values)


    def remove_entry(self, attribute, control):
        """
        Removes a specified entry from the UI and the corresponding attribute.

        Args:
            attribute: The attribute associated with the entry to be removed.
            control: The UI control representing the entry to be removed.
        """
        pm.deleteUI(control)
        pm.removeMultiInstance(attribute, b=True)

        self.enable_disable_add_button()

    def enable_disable_add_button(self):
        """
        Enables or disables the add button based on the number of used entries.
        """

        attribute_values = [attr_element.get() for attr_element in self.attribute]
        model_length = len(self.model)
        if len(attribute_values) >= model_length:
            pm.symbolButton(self.add_btn, edit=True, enable=False)
        else:
            pm.symbolButton(self.add_btn, edit=True, enable=True)

    def hydrate(self, model):
        """
        Updates the UI elements of the control to reflect the provided model data.

        Args:
            model: The data model containing host and plugin information to display in the UI.
        """
        self.model = model
        if not self.model:
            return

        for row in pm.columnLayout(self.column, q=True, childArray=True) or []:
            pm.deleteUI(row)

        for attr_element in self.attribute:
            self.create_row(attr_element)
        self.enable_disable_add_button()

    def on_menu_change(self, attribute, menu):
        """
        Callback function that updates the attribute when the selected plugin changes.

        Args:
            attribute: The attribute to update with the selected plugin's name.
            plugin_menu: The optionMenu control reflecting the plugin selection.
        """
        description = menu.getValue()
        model_entry = self._find_model_entry(description)
        if not model_entry:
            return
        attribute.set(model_entry["name"])

    def _find_model_entry(self, name_or_description):
        """
        Finds a model entry by either its name or description.

        Args:
            name_or_description (str): The name or description to search for

        Returns:
            dict: The matching model entry or None if not found
        """

        return next(
            (
                item
                for item in self.model
                if name_or_description in [item["name"], item["description"]]
            ),
            None,
        )


def _form_layout(*widgets, **kwargs):
    """
    Creates a form layout for a given set of widgets and applies standard spacing and attachments.

    Args:
        *widgets: Variable length widget list to include in the form layout.
        **kwargs: Additional keyword arguments (unused).

    Returns:
        The created form layout object.
    """
    form = pm.formLayout(nd=100)
    for widget in widgets:
        pm.control(widget, edit=True, parent=form)

    form.attachForm(widgets[0], "left", k.FORM_SPACING_X)
    form.attachControl(widgets[0], "right", k.FORM_SPACING_X, widgets[1])
    form.attachForm(widgets[0], "top", k.FORM_SPACING_Y)
    form.attachForm(widgets[0], "bottom", k.FORM_SPACING_Y)

    form.attachNone(widgets[1], "left")
    form.attachForm(widgets[1], "right", k.FORM_SPACING_X)
    form.attachForm(widgets[1], "top", k.FORM_SPACING_Y)
    form.attachForm(widgets[1], "bottom", k.FORM_SPACING_Y)

    return form
